/*
 * dta interfaces.  the attack code need to know these, and should use no
 * other information about the rest of the code.
 */

/*
 * Copyright (c) 2003 by Bennet Yee <bsy@cs.ucsd.edu>.
 * 
 * This material was developed by the author.  Permission to copy this
 * software, to redistribute it, and to use it for any purpose is hereby
 * granted, subject to the following five restrictions.
 * 
 * 1. Any copy made of this software must include this copyright notice in
 * full.
 * 
 * 2. All materials containing this software or derivatives thereof must
 * provide this software in source form or provide a means of obtaining
 * this software in source form; no fees may be charged for this software
 * except those for covering media costs and handling.
 * 
 * 3. All materials developed as a consequence of the use of this software
 * shall duly acknowledge such use, in accordance with the usual standards
 * of acknowledging credit in academic research.
 * 
 * 4. The author makes no warranty or representation that the operation
 * of this software will be error-free, and the author is under no
 * obligation to provide any services, by way of maintenance, update, or
 * otherwise.
 * 
 * 5. In conjunction with products arising from the use of this material,
 * there shall be no use of the names of the author, of the University of
 * California, nor of any adaptation thereof in any advertising,
 * promotional, or sales literature without prior written consent from the
 * author and the University of California in each case.
 * 
 * If you have questions, you can contact the author at bsy@cs.ucsd.edu
 * 
 * Users of this software is requested to make their best efforts (a) to
 * return to the author any improvements or extensions that they make, so
 * that these may be included in future releases; (b) to document clearly
 * any improvements or extensions made to this software; and (c) to inform
 * the author of noteworthy uses of this software.  Any improvements or
 * extensions to this software may be placed under different copyright
 * restrictions by the author of those improvements or extensions.  That
 * those improvements or extensions be placed under the same copyright as
 * this software is encouraged.
 */

#include <stdio.h>
#include <stdint.h>

#define	NLIMBS	4	/* 16 bytes == MD5_DIGEST_LENGTH; see model.c */

typedef struct big_int {
	uint32_t	val[NLIMBS];
} big_int_t;

typedef uint32_t	sim_time_t;

#define	SIM_TIME_MAX	(~0U)
#define	SIM_TIME_MIN	(0)

#define	MAXBITS	1024	/* set this to lower, e.g., 128, for testing */

typedef struct exponent {
	char		bits[MAXBITS];
	int		length;
} exp_t;

extern big_int_t const	*modulus;

int	big_int_equal(big_int_t const	*a,
		      big_int_t	const	*b);

void	big_int_print(big_int_t const	*var,
		      FILE		*iob);

void	modmult(big_int_t	*prod,
		sim_time_t	*cost,
		big_int_t const	*x,
		big_int_t const	*y,
		big_int_t const	*n);

void	modexp_z(big_int_t		*w,
		 big_int_t 		*zout,
		 sim_time_t		*tot,
		 big_int_t const	*x,
		 exp_t const		*e,
		 big_int_t const	*n);

void	modexp(big_int_t	*w,
	       sim_time_t	*tot,
	       big_int_t const	*x,
	       exp_t const	*e,
	       big_int_t const	*n);

void	bigrand(big_int_t	*r);

void	print_exp(exp_t const	*e,
		  FILE		*iob);

/*
 * w gets x^e mod n, where e is secret and n is public
 *
 * t gets "actual" time to compute w.  currently just computed from
 * timing model.
 */
void	query_card(big_int_t		*w,
		   sim_time_t		*t,
		   big_int_t const	*x);

extern int	verbose;

/* to be provided / filled in */

void	dta_preload(exp_t const	*provided);

void	dta(exp_t	*guess);
